@@ -1,13 +1,26 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007 Eli Carter <retracile@gmail.com>
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
+
 from genshi.builder import tag
 
-from trac.core import implements,Component
+from trac.core import Component, implements
+from trac.perm import IPermissionRequestor
 from trac.ticket.api import ITicketActionController
 from trac.ticket.default_workflow import ConfigurableTicketWorkflow
-from trac.perm import IPermissionRequestor
-from trac.config import Option, ListOption
 
-revision = "$Rev: 11075 $"
-url = "$URL: file:///var/svn/trac/tags/trac-1.0/sample-plugins/workflow/CodeReview.py $"
+revision = "$Rev: 13230 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/workflow/CodeReview.py $"
 
 class CodeReviewActionController(Component):
     """Support for simple code reviews.
@@ -37,12 +50,12 @@ class CodeReviewActionController(Component):
     implements(ITicketActionController, IPermissionRequestor)
 
     # IPermissionRequestor methods
-    
+
     def get_permission_actions(self):
         return ['TICKET_REVIEW']
 
     # ITicketActionController methods
-    
+
     def get_ticket_actions(self, req, ticket):
         # The review action is available in those status where it has been
         # configured, for those users who have the TICKET_REVIEW permission, as
@@ -54,7 +67,7 @@ class CodeReviewActionController(Component):
             controller = ConfigurableTicketWorkflow(self.env)
             actions_we_handle = controller.get_actions_by_operation_for_req(
                 req, ticket, 'code_review')
-        self.log.debug('code review handles actions: %r' % actions_we_handle)
+        self.log.debug('code review handles actions: %r', actions_we_handle)
         return actions_we_handle
 
     def get_all_status(self):
@@ -74,7 +87,7 @@ class CodeReviewActionController(Component):
         actions = ConfigurableTicketWorkflow(self.env).actions
 
         selected_value = grade or review_options[0][0]
-        
+
         label = actions[action]['name']
         control = tag(["as: ",
                        tag.select([tag.option(option, selected=
@@ -104,7 +117,7 @@ class CodeReviewActionController(Component):
     def _get_grade(self, req, action):
         id = action + '_code_review_result'
         return id, req.args.get(id)
-        
+
     def _get_review_options(self, action):
         return [[x.strip() for x in raw_option.split('->')]
                 for raw_option in self.config.getlist('ticket-workflow',
@@ -116,4 +129,4 @@ class CodeReviewActionController(Component):
             review_options = self._get_review_options(action)
         for option, status in review_options:
             if grade == option:
-                return status            
+                return status
