@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright (C) 2003-2010 Edgewall Software
 # All rights reserved.
 #
@@ -12,8 +12,9 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
+from __future__ import with_statement
+
 import cmd
-import locale
 import os.path
 import pkg_resources
 from shlex import shlex
@@ -22,39 +23,40 @@ import sys
 import traceback
 
 from trac import __version__ as VERSION
-from trac.admin import AdminCommandError, AdminCommandManager
+from trac.admin.api import AdminCommandError, AdminCommandManager, \
+                           get_console_locale
 from trac.core import TracError
 from trac.env import Environment
 from trac.ticket.model import *
-from trac.util import translation
+from trac.util import translation, warn_setuptools_issue
 from trac.util.html import html
 from trac.util.text import console_print, exception_to_unicode, printout, \
-                           printerr, raw_input, to_unicode
-from trac.util.translation import _, get_negotiated_locale, has_babel
+                           printerr, raw_input, to_unicode, \
+                           getpreferredencoding
+from trac.util.translation import _, ngettext, has_babel, cleandoc_
 from trac.versioncontrol.api import RepositoryManager
 from trac.wiki.admin import WikiAdmin
+from trac.wiki.formatter import MacroError
 from trac.wiki.macros import WikiMacroBase
 
+
 TRAC_VERSION = pkg_resources.get_distribution('Trac').version
 rl_completion_suppress_append = None
-LANG = os.environ.get('LANG')
+
 
 def find_readline_lib():
     """Return the name (and possibly the full path) of the readline library
     linked to the readline module.
     """
     import readline
-    f = open(readline.__file__, "rb")
-    try:
+    with open(readline.__file__, "rb") as f:
         data = f.read()
-    finally:
-        f.close()
     import re
     m = re.search('\0([^\0]*libreadline[^\0]*)\0', data)
     if m:
         return m.group(1)
     return None
-    
+
 
 class TracAdmin(cmd.Cmd):
     intro = ''
@@ -65,6 +67,7 @@ class TracAdmin(cmd.Cmd):
     prompt = "Trac> "
     envname = None
     __env = None
+    needs_upgrade = None
 
     def __init__(self, envdir=None):
         cmd.Cmd.__init__(self)
@@ -74,7 +77,7 @@ class TracAdmin(cmd.Cmd):
             for c in '-/:()\\':
                 delims = delims.replace(c, '')
             readline.set_completer_delims(delims)
-            
+
             # Work around trailing space automatically inserted by libreadline
             # until Python gets fixed, see http://bugs.python.org/issue5833
             import ctypes
@@ -100,7 +103,7 @@ class TracAdmin(cmd.Cmd):
                 if self.interactive:
                     encoding = sys.stdin.encoding
                 else:
-                    encoding = locale.getpreferredencoding() # sys.argv
+                    encoding = getpreferredencoding() # sys.argv
                 line = to_unicode(line, encoding)
             if self.interactive:
                 line = line.replace('\\', '\\\\')
@@ -129,7 +132,7 @@ class TracAdmin(cmd.Cmd):
         self.interactive = True
         printout(_("""Welcome to trac-admin %(version)s
 Interactive Trac administration console.
-Copyright (C) 2003-2012 Edgewall Software
+Copyright (C) 2003-2013 Edgewall Software
 
 Type:  '?' or 'help' for help on commands.
         """, version=TRAC_VERSION))
@@ -149,7 +152,7 @@ Type:  '?' or 'help' for help on commands.
         if not self.__env:
             try:
                 self._init_env()
-            except:
+            except Exception:
                 return False
         return True
 
@@ -166,17 +169,21 @@ Type:  '?' or 'help' for help on commands.
 
     def _init_env(self):
         self.__env = env = Environment(self.envname)
+        negotiated = None
         # fixup language according to env settings
         if has_babel:
-            default = env.config.get('trac', 'default_language', '')
-            negotiated = get_negotiated_locale([LANG, default])
+            negotiated = get_console_locale(env)
             if negotiated:
                 translation.activate(negotiated)
-        
+
     ##
     ## Utility methods
     ##
 
+    @property
+    def cmd_mgr(self):
+        return AdminCommandManager(self.env)
+
     def arg_tokenize(self, argstr):
         """`argstr` is an `unicode` string
 
@@ -201,7 +208,7 @@ Type:  '?' or 'help' for help on commands.
         paragraphs = re.split(r'(?m)(?:^[ \t]*\n){1,}', text)
         return [re.sub(r'(?m)\s+', ' ', each.strip())
                 for each in paragraphs]
-    
+
     @classmethod
     def print_doc(cls, docs, stream=None, short=False, long=False):
         if stream is None:
@@ -213,7 +220,7 @@ Type:  '?' or 'help' for help on commands.
             max_len = max(len(doc[0]) for doc in docs)
             for (cmd, args, doc) in docs:
                 paragraphs = cls.split_help_text(doc)
-                console_print(stream, '%s  %s' % (cmd.ljust(max_len), 
+                console_print(stream, '%s  %s' % (cmd.ljust(max_len),
                                                   paragraphs[0]))
         else:
             import textwrap
@@ -223,14 +230,14 @@ Type:  '?' or 'help' for help on commands.
                 console_print(stream, '    %s\n' % paragraphs[0])
                 if (long or len(docs) == 1) and len(paragraphs) > 1:
                     for paragraph in paragraphs[1:]:
-                        console_print(stream, textwrap.fill(paragraph, 79, 
+                        console_print(stream, textwrap.fill(paragraph, 79,
                             initial_indent='    ', subsequent_indent='    ')
                             + '\n')
 
     ##
     ## Command dispatcher
     ##
-    
+
     def complete_line(self, text, line, cmd_only=False):
         if rl_completion_suppress_append is not None:
             rl_completion_suppress_append.value = 1
@@ -238,9 +245,8 @@ Type:  '?' or 'help' for help on commands.
         if line and line[-1] == ' ':    # Space starts new argument
             args.append('')
         if self.env_check():
-            cmd_mgr = AdminCommandManager(self.env)
             try:
-                comp = cmd_mgr.complete_command(args, cmd_only)
+                comp = self.cmd_mgr.complete_command(args, cmd_only)
             except Exception, e:
                 printerr()
                 printerr(_('Completion error: %(err)s',
@@ -255,24 +261,31 @@ Type:  '?' or 'help' for help on commands.
             return comp.complete(text)
         except AttributeError:
             return self.word_complete(text, comp)
-        
+
     def completenames(self, text, line, begidx, endidx):
         return self.complete_line(text, line, True)
-        
+
     def completedefault(self, text, line, begidx, endidx):
         return self.complete_line(text, line)
-        
+
     def default(self, line):
         try:
             if not self.__env:
                 self._init_env()
+            if self.needs_upgrade is None:
+                self.needs_upgrade = self.__env.needs_upgrade()
         except TracError, e:
             raise AdminCommandError(to_unicode(e))
         except Exception, e:
             raise AdminCommandError(exception_to_unicode(e))
         args = self.arg_tokenize(line)
-        cmd_mgr = AdminCommandManager(self.env)
-        return cmd_mgr.execute_command(*args)
+        if args[0] == 'upgrade':
+            self.needs_upgrade = None
+        elif self.needs_upgrade:
+            raise TracError(_('The Trac Environment needs to be upgraded.\n\n'
+                              'Run "trac-admin %(path)s upgrade"',
+                              path=self.envname))
+        return self.cmd_mgr.execute_command(*args)
 
     ##
     ## Available Commands
@@ -290,19 +303,31 @@ Type:  '?' or 'help' for help on commands.
 
     def complete_help(self, text, line, begidx, endidx):
         return self.complete_line(text, line[5:], True)
-        
+
     def do_help(self, line=None):
         arg = self.arg_tokenize(line)
         if arg[0]:
+            cmd_mgr = None
             doc = getattr(self, "_help_" + arg[0], None)
             if doc is None and self.env_check():
-                cmd_mgr = AdminCommandManager(self.env)
+                cmd_mgr = self.cmd_mgr
                 doc = cmd_mgr.get_command_help(arg)
             if doc:
                 self.print_doc(doc)
             else:
-                printerr(_("No documentation found for '%(cmd)s'",
+                printerr(_("No documentation found for '%(cmd)s'."
+                           " Use 'help' to see the list of commands.",
                            cmd=' '.join(arg)))
+                cmds = None
+                if cmd_mgr:
+                    cmds = cmd_mgr.get_similar_commands(arg[0])
+                if cmds:
+                    printout('')
+                    printout(ngettext("Did you mean this?",
+                                      "Did you mean one of these?",
+                                      len(cmds)))
+                    for cmd in cmds:
+                        printout('    ' + cmd)
         else:
             printout(_("trac-admin - The Trac Administration Console "
                        "%(version)s", version=TRAC_VERSION))
@@ -313,7 +338,7 @@ Type:  '?' or 'help' for help on commands.
                     )
                 printout(_("Invoking trac-admin without command starts "
                            "interactive mode.\n"))
-            env = self.env_check() and self.env or None
+            env = self.env if self.env_check() else None
             self.print_doc(self.all_docs(env), short=True)
 
 
@@ -334,10 +359,10 @@ Type:  '?' or 'help' for help on commands.
     _help_initenv = [
         ('initenv', '[<projectname> <db> [<repostype> <repospath>]]',
          """Create and initialize a new environment
-         
+
          If no arguments are given, then the required parameters are requested
          interactively.
-         
+
          One or more optional arguments --inherit=PATH can be used to specify
          the "[inherit] file" option at environment creation time, so that only
          the options not already specified in one of the global configuration
@@ -354,7 +379,7 @@ Type:  '?' or 'help' for help on commands.
         printout(_("Creating a new Trac environment at %(envname)s",
                    envname=self.envname))
         printout(_("""
-Trac will first ask a few questions about your environment 
+Trac will first ask a few questions about your environment
 in order to initialize and prepare the project database.
 
  Please enter the name of your project.
@@ -363,10 +388,10 @@ in order to initialize and prepare the project database.
         dp = 'My Project'
         returnvals.append(raw_input(_("Project Name [%(default)s]> ",
                                       default=dp)).strip() or dp)
-        printout(_(""" 
+        printout(_("""
  Please specify the connection string for the database to use.
  By default, a local SQLite database is created in the environment
- directory. It is also possible to use an already existing
+ directory. It is also possible to use an existing MySQL or
  PostgreSQL database (check the Trac documentation for the exact
  connection string syntax).
 """))
@@ -392,7 +417,7 @@ in order to initialize and prepare the project database.
             initenv_error(_("Base directory '%(env)s' does not exist. Please "
                             "create it manually and retry.",
                             env=os.path.dirname(self.envname)))
-            return 2            
+            return 2
 
         arg = self.arg_tokenize(line)
         inherit_paths = []
@@ -443,8 +468,8 @@ in order to initialize and prepare the project database.
 
             # Add a few default wiki pages
             printout(_(" Installing default wiki pages"))
-            pages_dir = pkg_resources.resource_filename('trac.wiki', 
-                                                        'default-pages') 
+            pages_dir = pkg_resources.resource_filename('trac.wiki',
+                                                        'default-pages')
             WikiAdmin(self.__env).load_pages(pages_dir)
 
             if repository_dir:
@@ -458,12 +483,12 @@ in order to initialize and prepare the project database.
 ---------------------------------------------------------------------
 Warning: couldn't index the default repository.
 
-This can happen for a variety of reasons: wrong repository type, 
+This can happen for a variety of reasons: wrong repository type,
 no appropriate third party library for this repository type,
 no actual repository at the specified repository path...
 
-You can nevertheless start using your Trac environment, but 
-you'll need to check again your trac.ini file and the [trac] 
+You can nevertheless start using your Trac environment, but
+you'll need to check again your trac.ini file and the [trac]
 repository_type and repository_path settings.
 """))
         except Exception, e:
@@ -502,9 +527,11 @@ Congratulations!
     def _resync_feedback(self, rev):
         sys.stdout.write(' [%s]\r' % rev)
         sys.stdout.flush()
-        
+
 
 class TracAdminHelpMacro(WikiMacroBase):
+    _domain = 'messages'
+    _description = cleandoc_(
     """Display help for trac-admin commands.
 
     Examples:
@@ -514,7 +541,7 @@ class TracAdminHelpMacro(WikiMacroBase):
     [[TracAdminHelp(wiki export)]]  # the "wiki export" command
     [[TracAdminHelp(upgrade)]]      # the upgrade command
     }}}
-    """
+    """)
 
     def expand_macro(self, formatter, name, content):
         if content:
@@ -524,30 +551,31 @@ class TracAdminHelpMacro(WikiMacroBase):
                 cmd_mgr = AdminCommandManager(self.env)
                 doc = cmd_mgr.get_command_help(arg)
             if not doc:
-                raise TracError('Unknown trac-admin command "%s"' % content)
+                raise MacroError(_('Unknown trac-admin command '
+                                   '"%(command)s"', command=content))
         else:
             doc = TracAdmin.all_docs(self.env)
         buf = StringIO.StringIO()
         TracAdmin.print_doc(doc, buf, long=True)
-        return html.PRE(buf.getvalue(), class_='wiki')
+        return html.PRE(buf.getvalue().decode('utf-8'), class_='wiki')
 
 
-def run(args=None):
-    """Main entry point."""
+def _quote_args(args):
+    def quote(arg):
+        if arg.isalnum():
+            return arg
+        return '"\'"'.join("'%s'" % v for v in arg.split("'"))
+    return [quote(arg) for arg in args]
+
+
+def _run(args):
     if args is None:
         args = sys.argv[1:]
-    locale = None
-    if has_babel:
-        import babel
-        try:
-            locale = get_negotiated_locale([LANG]) or babel.Locale.default()
-        except babel.UnknownLocaleError:
-            pass
-        translation.activate(locale)
+    warn_setuptools_issue()
     admin = TracAdmin()
     if len(args) > 0:
         if args[0] in ('-h', '--help', 'help'):
-            return admin.onecmd(' '.join(['help'] + args[1:]))
+            return admin.onecmd(' '.join(_quote_args(['help'] + args[1:])))
         elif args[0] in ('-v','--version'):
             printout(os.path.basename(sys.argv[0]), TRAC_VERSION)
         else:
@@ -557,12 +585,10 @@ def run(args=None):
             except UnicodeDecodeError:
                 printerr(_("Non-ascii environment path '%(path)s' not "
                            "supported.", path=to_unicode(env_path)))
-                sys.exit(2)
+                return 2
             admin.env_set(env_path)
             if len(args) > 1:
-                s_args = ' '.join(["'%s'" % c for c in args[2:]])
-                command = args[1] + ' ' +s_args
-                return admin.onecmd(command)
+                return admin.onecmd(' '.join(_quote_args(args[1:])))
             else:
                 while True:
                     try:
@@ -573,6 +599,15 @@ def run(args=None):
         return admin.onecmd("help")
 
 
+def run(args=None):
+    """Main entry point."""
+    translation.activate(get_console_locale())
+    try:
+        return _run(args)
+    finally:
+        translation.deactivate()
+
+
 if __name__ == '__main__':
     pkg_resources.require('Trac==%s' % VERSION)
     sys.exit(run())
