@@ -110,6 +110,10 @@ def _markup_to_unicode(markup):
     return to_unicode(markup)
 
 
+class MacroError(TracError):
+    pass
+
+
 class ProcessorError(TracError):
     pass
 
@@ -329,14 +333,14 @@ class WikiProcessor(object):
     # generic processors
 
     def _legacy_macro_processor(self, text): # TODO: remove in 0.12
-        self.env.log.warning('Executing pre-0.11 Wiki macro %s by provider %s'
-                             % (self.name, self.macro_provider))
+        self.env.log.warning('Executing pre-0.11 Wiki macro %s by '
+                             'provider %s', self.name, self.macro_provider)
         return self.macro_provider.render_macro(self.formatter.req, self.name,
                                                 text)
 
     def _macro_processor(self, text):
-        self.env.log.debug('Executing Wiki macro %s by provider %s'
-                           % (self.name, self.macro_provider))
+        self.env.log.debug('Executing Wiki macro %s by provider %s',
+                           self.name, self.macro_provider)
         if arity(self.macro_provider.expand_macro) == 4:
             return self.macro_provider.expand_macro(self.formatter, self.name,
                                                     text, self.args)
@@ -724,7 +728,8 @@ class Formatter(object):
         interwiki = InterWikiMap(self.env)
         if ns in interwiki:
             url, title = interwiki.url(ns, target)
-            return self._make_ext_link(url, label, title)
+            if url:
+                return self._make_ext_link(url, label, title)
 
     def _make_ext_link(self, url, text, title=''):
         local_url = self.env.project_url or \
@@ -786,6 +791,8 @@ class Formatter(object):
             args = fullmatch.group('macroargs')
         try:
             return macro.ensure_inline(macro.process(args))
+        except MacroError, e:
+            return system_message(e)
         except Exception, e:
             self.env.log.error('Macro %s(%s) failed:%s', name, args,
                                exception_to_unicode(e, traceback=True))
@@ -861,12 +868,12 @@ class Formatter(object):
         else:
             type_ = 'ol'
             lstart = fullmatch.group('lstart')
-            start = None
-            idx = '0iI'.find(listid)
-            if idx > -1:
-                class_ = ('arabiczero', 'lowerroman', 'upperroman')[idx]
-            elif listid.isdigit():
-                start = lstart != '1' and int(lstart)
+            if listid == 'i':
+                class_ = 'lowerroman'
+            elif listid == 'I':
+                class_ = 'upperroman'
+            elif listid.isdigit() and lstart != '1':
+                start = int(lstart)
             elif listid.islower():
                 class_ = 'loweralpha'
                 if len(lstart) == 1 and lstart != 'a':
@@ -892,7 +899,7 @@ class Formatter(object):
             self._list_stack.append((new_type, depth))
             self._set_tab(depth)
             class_attr = ' class="%s"' % lclass if lclass else ''
-            start_attr = ' start="%s"' % start if start else ''
+            start_attr = ' start="%s"' % start if start is not None else ''
             self.out.write('<' + new_type + class_attr + start_attr + '><li>')
         def close_item():
             self.flush_tags()
